#include "precomp.h"

inline bool operator==(const CONNECTDATA& cd, DWORD dwCookie) { return(dwCookie == cd.dwCookie); }

CConnectionPoint::~CConnectionPoint()
{
    CONNECTDATA cd = {};

    while(m_Connections.Remove(0, &cd))
    {
        if(cd.pUnk)
            cd.pUnk->Release();
    }
}

IFACEMETHODIMP
CConnectionPoint::QueryInterface(REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = nullptr;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_IConnectionPoint))
            *ppvObject = static_cast<IConnectionPoint*>(this);

        if(*ppvObject)
        {
            static_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

IFACEMETHODIMP_(ULONG)
CConnectionPoint::AddRef()
{
    return(m_pCPC->AddRef());
}

IFACEMETHODIMP_(ULONG)
CConnectionPoint::Release()
{
    return(m_pCPC->Release());
}

IFACEMETHODIMP
CConnectionPoint::GetConnectionPointContainer(IConnectionPointContainer** ppCPC)
{
    return(m_pCPC->QueryInterface(IID_IConnectionPointContainer, reinterpret_cast<void**>(ppCPC)));
}

IFACEMETHODIMP
CConnectionPoint::Advise(IUnknown* pUnkSink, DWORD* pdwCookie)
{
    CONNECTDATA data = {};
    HRESULT     hr;
    DWORD       dwCookie = 1;
    IID         iid;

    if(IsBadWritePtrT(pdwCookie))
        return(E_POINTER);

    *pdwCookie = 0;

    if(FAILED(hr = GetConnectionInterface(&iid)))
        return(hr);

    if(m_Connections.Get(m_Connections.Size() - 1, &data))
        dwCookie = data.dwCookie + 1;

    if(!dwCookie)
    {
        m_Connections.Enum([&dwCookie](const CONNECTDATA& cd) -> bool
        {
            return(cd.dwCookie <= ++dwCookie);
        });

        if(!dwCookie)
            return(CONNECT_E_ADVISELIMIT);
    }

    data.dwCookie = dwCookie;
    data.pUnk = nullptr;

    if(SUCCEEDED(hr = TryConnect(pUnkSink, iid, reinterpret_cast<void**>(&data.pUnk))))
    {
        if(m_Connections.Insert(dwCookie - 1, data))
        {
            *pdwCookie = data.dwCookie;
        }
        else
        {
            hr = E_OUTOFMEMORY;
            data.pUnk->Release();
        }
    }

    return(hr);
}

IFACEMETHODIMP
CConnectionPoint::Unadvise(DWORD dwCookie)
{
    CONNECTDATA cd;

    if(m_Connections.FindAndRemove(dwCookie, &cd))
    {
        cd.pUnk->Release();
        return(S_OK);
    }

    return(CONNECT_E_NOCONNECTION);
}

IFACEMETHODIMP
CConnectionPoint::EnumConnections(IEnumConnections** ppEnum)
{
    CEnumConnections* pEnum;

    if(IsBadWritePtrT(ppEnum))
        return(E_POINTER);

    *ppEnum = nullptr;

    if(nullptr == (pEnum = new (std::nothrow) CEnumConnections(this)))
        return(E_OUTOFMEMORY);

    const HRESULT hr = pEnum->QueryInterface(IID_IEnumConnections, reinterpret_cast<void**>(ppEnum));
    pEnum->Release();
    return(hr);
}

__checkReturn HRESULT
CConnectionPoint::GetConnectData(__in UINT_PTR uIndex, __out CONNECTDATA* pData)
{
    if(!m_Connections.Get(uIndex, pData))
        return(HRESULT_FROM_WIN32(GetLastError()));

    pData->pUnk->AddRef();
    return(S_OK);
}

__checkReturn HRESULT
CConnectionPoint::TryConnect(__in IUnknown* pUnk, __in REFIID riid, __deref_out void** ppvObject)
{
    __try
    {
        if(FAILED(pUnk->QueryInterface(riid, ppvObject)))
            return(CONNECT_E_CANNOTCONNECT);
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_POINTER);
    }

    return((*ppvObject) ? S_OK : E_UNEXPECTED);
}
