#include "precomp.h"

CDeviceList::CDeviceList()
    : m_hWnd(nullptr), m_dpiX(USER_DEFAULT_SCREEN_DPI), m_dpiY(USER_DEFAULT_SCREEN_DPI)
{
    m_Sink.SetParent(this);
    m_Sink.Connect();
}

HRESULT
CDeviceList::DoModal(HWND hWndParent)
{
    CThreadDpiAwarenessContext dpiAwarness(DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE_V2);
    CDeviceList dlg;
    CActCtx ctx(IDR_MANIFEST);
    const INT_PTR n = DialogBoxParam(g_hInst, MAKEINTRESOURCE(IDD_DEVICES), hWndParent, DlgProc, reinterpret_cast<LPARAM>(&dlg));
    return((n == -1) ? HRESULT_FROM_WIN32(GetLastError()) : ((n == IDOK) ? S_OK : S_FALSE));
}

INT_PTR
CDeviceList::OnWmInitDialog(HWND hWnd, WPARAM, LPARAM)
{
    LVCOLUMN lvc = {};
    RECT rc;
    TCHAR sz[64];
    HWND hWndLV = GetDlgItem(hWnd, IDC_DEVICES);

    m_hWnd = hWnd;
    m_dpiX = m_dpiY = MyGetDpiForWindow(hWnd);

    GetClientRect(hWnd, &rc);
    OnWmSize(hWnd, SIZE_RESTORED, MAKELPARAM(rc.right - rc.left, rc.bottom - rc.top));
    GetClientRect(hWndLV, &rc);

    lvc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT;
    lvc.fmt = LVCFMT_LEFT;
    lvc.cx = ((rc.right - rc.left) - MyGetSystemMetricsForDpi(SM_CXVSCROLL, m_dpiX)) / 4;
    lvc.pszText = sz;

    for(UINT idx = IDS_TARGETVOLTAGE; idx >= IDS_DEVICE; --idx)
        if(LoadString(g_hInst, idx, sz, _countof(sz)))
            ListView_InsertColumn(hWndLV, 0, &lvc);

    m_Sink.EnumDevices();
    SetWindowTheme(hWndLV, L"Explorer", nullptr);
    ListView_SetExtendedListViewStyle(hWndLV, LVS_EX_FULLROWSELECT | LVS_EX_DOUBLEBUFFER);
    return(TRUE);
}

INT_PTR
CDeviceList::OnWmCommand(HWND hWnd, WPARAM wParam, LPARAM)
{
    LVITEMA lvi;

    switch(LOWORD(wParam))
    {
        case IDOK:
            if(!IsWindowEnabled(GetDlgItem(hWnd, IDOK)))
                break;

            if(0 > (lvi.iItem = static_cast<int>(SendDlgItemMessage(hWnd, IDC_DEVICES, LVM_GETNEXTITEM, static_cast<WPARAM>(-1), LVNI_SELECTED))))
                break;

            lvi.iSubItem = 0;
            lvi.cchTextMax = _countof(MonConf.idDevice);
            lvi.pszText = MonConf.idDevice;
            SendDlgItemMessage(hWnd, IDC_DEVICES, LVM_GETITEMTEXTA, static_cast<WPARAM>(lvi.iItem), reinterpret_cast<LPARAM>(&lvi));
        case IDCANCEL:
            EndDialog(hWnd, LOWORD(wParam));
            break;
        default:
            return(FALSE);
    }

    return(TRUE);
}

INT_PTR
CDeviceList::OnWmNotify(HWND hWnd, WPARAM, LPARAM lParam)
{
    NMLISTVIEW* pnmlv = reinterpret_cast<NMLISTVIEW*>(lParam);

    switch(reinterpret_cast<NMHDR*>(lParam)->code)
    {
        case LVN_ITEMCHANGING:
            if((pnmlv->uChanged & LVIF_STATE) && (pnmlv->uOldState & LVIS_SELECTED) && (!(pnmlv->uNewState & LVIS_SELECTED)))
                EnableDlgItem(hWnd, IDOK, FALSE);

            return(TRUE);
        case LVN_ITEMCHANGED:
            if((pnmlv->uChanged & LVIF_STATE) && (!(pnmlv->uOldState & LVIS_SELECTED)) && (pnmlv->uNewState & LVIS_SELECTED))
                EnableDlgItem(hWnd, IDOK, TRUE);

            return(TRUE);
        case LVN_ITEMACTIVATE:
            PostMessage(hWnd, WM_COMMAND, MAKEWPARAM(IDOK, BN_CLICKED), reinterpret_cast<LPARAM>(GetDlgItem(hWnd, IDOK)));
            return(TRUE);
        case LVN_GETEMPTYMARKUP:
            reinterpret_cast<NMLVEMPTYMARKUP*>(lParam)->dwFlags = EMF_CENTERED;
            SetWindowLongPtr(hWnd, DWLP_MSGRESULT, static_cast<LONG_PTR>(0 != LoadString(g_hInst, IDS_NODEVICEATTACHED, reinterpret_cast<NMLVEMPTYMARKUP*>(lParam)->szMarkup, _countof(reinterpret_cast<NMLVEMPTYMARKUP*>(lParam)->szMarkup))));
            return(TRUE);
        case LVN_DELETEITEM:
            reinterpret_cast<CSmartICE04EventsSink*>(pnmlv->lParam)->Disconnect();
            reinterpret_cast<CSmartICE04EventsSink*>(pnmlv->lParam)->Release();
            return(TRUE);
        default:
            break;
    }

    return(FALSE);
}

INT_PTR
CDeviceList::OnWmSize(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
    if(wParam != SIZE_MINIMIZED)
    {
        RECT rc;
        SIZE margin;

        GetWindowRect(GetDlgItem(hWnd, IDC_DEVICES), &rc);
        RectScreenToClient(hWnd, &rc);
        margin.cx = rc.left;
        margin.cy = rc.top;

        HWND hCtl = GetDlgItem(hWnd, IDCANCEL);
        GetWindowRect(hCtl, &rc);

        rc.right -= rc.left;
        rc.bottom -= rc.top;
        rc.left = LOWORD(lParam) - (margin.cx + rc.right);
        rc.top = HIWORD(lParam) - (margin.cy + rc.bottom);

        CDeferWindowPos pos(3);
        pos.MoveWindow(hCtl, rc.left, rc.top, rc.right, rc.bottom, TRUE);
        pos.MoveWindow(GetDlgItem(hWnd, IDOK), rc.left - (margin.cx + rc.right), rc.top, rc.right, rc.bottom, TRUE);
        pos.MoveWindow(GetDlgItem(hWnd, IDC_DEVICES), margin.cx, margin.cy, LOWORD(lParam) - (2 * margin.cx), rc.top - (2 * margin.cy), TRUE);
    }

    return(TRUE);
}

INT_PTR
CDeviceList::OnWmDpiChanged(HWND hWnd, WPARAM wParam, LPARAM)
{
    ISmartICE04*    pDevice;
    HIMAGELIST      himl;
    LVITEM          lvi = {};
    HWND            hWndLV = GetDlgItem(hWnd, IDC_DEVICES);

    const UINT dpiX = LOWORD(wParam);
    const UINT dpiY = HIWORD(wParam);

    while(m_IconIndex.Remove(0))
        ;

    if(nullptr != (himl = ListView_SetImageList(hWndLV, nullptr, LVSIL_SMALL)))
        ImageList_Destroy(himl);

    ListView_ScaleColumnHeaders(hWndLV, m_dpiX, dpiX);
    m_dpiX = dpiX;
    m_dpiY = dpiY;

    for(lvi.iItem = 0;; ++lvi.iItem)
    {
        lvi.mask = LVIF_PARAM;

        if(!ListView_GetItem(hWndLV, &lvi))
            break;

        if(!lvi.lParam)
            continue;

        if(SUCCEEDED(reinterpret_cast<CSmartICE04EventsSink*>(lvi.lParam)->GetDevice(&pDevice)))
        {
            lvi.mask = LVIF_IMAGE;
            lvi.iImage = AddIcon(pDevice);
            ListView_SetItem(hWndLV, &lvi);
            pDevice->Release();
        }
    }

    return(TRUE);
}

INT_PTR CALLBACK
CDeviceList::DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    static const DLG_MSGMAP<CDeviceList> sc_map[] =
    {
        { WM_SIZE,          &CDeviceList::OnWmSize          },
        { WM_NOTIFY,        &CDeviceList::OnWmNotify        },
        { WM_COMMAND,       &CDeviceList::OnWmCommand       },
        { WM_DPICHANGED,    &CDeviceList::OnWmDpiChanged    }
    };

    for(uintptr_t n = 0; n < _countof(sc_map); ++n)
    {
        if(sc_map[n].uMsg == uMsg)
        {
            CDeviceList* pThis = reinterpret_cast<CDeviceList*>(GetWindowLongPtr(hWnd, DWLP_USER));

            if(pThis)
                return((pThis->*sc_map[n].pfn)(hWnd, wParam, lParam));
        }
    }

    if(uMsg == WM_INITDIALOG)
    {
        SetWindowLongPtr(hWnd, DWLP_USER, lParam);
        return(reinterpret_cast<CDeviceList*>(lParam)->OnWmInitDialog(hWnd, wParam, lParam));
    }

    return(FALSE);
}

void
CDeviceList::AddDevice(ISmartICE04* pDevice)
{
    LVFINDINFO lvfi;
    LVITEM lvi;
    VARIANT_BOOL vbl;
    ChipId id;
    long voltage;
    BSTR sernum = nullptr;
    HWND hWndLV = GetDlgItem(m_hWnd, IDC_DEVICES);

    if(FAILED(pDevice->get_SerialNumber(&sernum)))
        return;

    lvfi.flags = LVFI_STRING;
    lvfi.psz = sernum;

    if(0 > ListView_FindItem(hWndLV, -1, &lvfi))
    {
        CSmartICE04EventsSink* pSink = new (std::nothrow) CSmartICE04EventsSink(*this);

        if(pSink)
        {
            pSink->Connect(pDevice);
            lvi.mask = LVIF_IMAGE | LVIF_PARAM | LVIF_TEXT;
            lvi.iItem = MAXLONG;
            lvi.iSubItem = 0;
            lvi.pszText = sernum;
            lvi.iImage = AddIcon(pDevice);
            lvi.lParam = reinterpret_cast<LPARAM>(pSink);

            if(0 <= ListView_InsertItem(hWndLV, &lvi))
            {
                pDevice->get_IsInDebugMode(&vbl);
                pSink->OnDebugModeChanged(pDevice, vbl);
                pDevice->get_ChipId(&id);
                pSink->OnTargetChanged(pDevice, id);
                pDevice->get_TargetVoltage(&voltage);
                pSink->OnTargetVoltageChanged(pDevice, (voltage >= 1800) ? VARIANT_TRUE : VARIANT_FALSE);

                // Remove markup previously set through LVN_GETEMPTYMARKUP
                if(1 == ListView_GetItemCount(hWndLV))
                    InvalidateRect(hWndLV, nullptr, FALSE);
            }
            else
            {
                pSink->Disconnect();
                pSink->Release();
            }
        }
    }

    SysFreeString(sernum);
}

void
CDeviceList::RemoveDevice(ISmartICE04* pDevice)
{
    int iItem;

    if(0 <= (iItem = FindDevice(pDevice)))
        ListView_DeleteItem(GetDlgItem(m_hWnd, IDC_DEVICES), iItem);
}

void
CDeviceList::SetTargetVoltage(ISmartICE04* pDevice, LPCTSTR pszVoltage)
{
    SetSubItemText(pDevice, 3, pszVoltage);
}

void
CDeviceList::SetTarget(ISmartICE04* pDevice, LPCTSTR pszTarget)
{
    SetSubItemText(pDevice, 2, pszTarget);
}

void
CDeviceList::SetDebugMode(ISmartICE04* pDevice, LPCTSTR pszDebugMode)
{
    SetSubItemText(pDevice, 1, pszDebugMode);
}

void
CDeviceList::SetSubItemText(ISmartICE04* pDevice, int iSubItem, LPCTSTR psz)
{
    LVITEM lvi;

    if(0 > (lvi.iItem = FindDevice(pDevice)))
        return;

    lvi.mask = LVIF_TEXT;
    lvi.iSubItem = iSubItem;
    lvi.pszText = const_cast<LPTSTR>(psz);
    ListView_SetItem(GetDlgItem(m_hWnd, IDC_DEVICES), &lvi);
}

int
CDeviceList::FindDevice(ISmartICE04* pDevice) const
{
    LVFINDINFO lvfi;
    BSTR bstr = nullptr;

    if(FAILED(pDevice->get_SerialNumber(&bstr)))
        return(-1);

    lvfi.flags = LVFI_STRING;
    lvfi.psz = bstr;
    const int idx = ListView_FindItem(GetDlgItem(m_hWnd, IDC_DEVICES), -1, &lvfi);
    SysFreeString(bstr);
    return(idx);
}

int
CDeviceList::AddIcon(ISmartICE04* pDevice)
{
    HIMAGELIST  himl;
    ICONINDEX   ii;
    HWND        hWndLV = GetDlgItem(m_hWnd, IDC_DEVICES);

    if(FAILED(pDevice->get_ProductId(&ii.id)))
        return(-1);

    if(m_IconIndex.Find(ii.id, &ii))
        return(ii.idx);

    const UINT dpi = MyGetDpiForWindow(m_hWnd);

    if(nullptr == (himl = ListView_GetImageList(hWndLV, LVSIL_SMALL)))
    {
        if(nullptr == (himl = ImageList_Create(MinGetSystemMetricsForDpi(SM_CXSMICON, SM_CXMENUCHECK, dpi), MinGetSystemMetricsForDpi(SM_CYSMICON, SM_CYMENUCHECK, dpi), ILC_COLOR32 | ILC_MASK, 1, 63)))
            return(-1);

        ListView_SetImageList(hWndLV, himl, LVSIL_SMALL);
    }

    IPictureDisp* pDispatch = nullptr;

    if(FAILED(pDevice->get_SmallIcon(&pDispatch)))
        return(-1);

    ii.idx = -1;

    if(pDispatch)
    {
        IPicture2* pPicture = nullptr;

        if(FAILED(pDispatch->QueryInterface(IID_IPicture2, reinterpret_cast<void**>(&pPicture))))
        {
            pPicture = nullptr;

            if(FAILED(pDispatch->QueryInterface(IID_IPicture, reinterpret_cast<void**>(&pPicture))))
                pPicture = nullptr;
        }

        if(pPicture)
        {
            SHORT pictype = PICTYPE_NONE;

            if(SUCCEEDED(pPicture->get_Type(&pictype)))
            {
                if(pictype == PICTYPE_ICON)
                {
                    HICON hIcon = nullptr;

                    if(SUCCEEDED(pPicture->get_Handle(reinterpret_cast<HHANDLE*>(&hIcon))))
                    {
                        if(0 <= (ii.idx = ImageList_AddIcon(himl, hIcon)))
                        {
                            m_IconIndex.Add(ii);
                        }
                    }
                }
            }
            pPicture->Release();
        }
        pDispatch->Release();
    }

    return(ii.idx);
}
