#ifndef CLIST_H
#define CLIST_H

template<typename T>
class CList
{
public:
    CList() : m_pHead(nullptr), m_pTail(nullptr), m_uSize(0) {}

    ~CList()
    {
        while(Remove(0))
            ;
    }

    _Success_(return != FALSE)
    BOOL Add(__in const T& val) { return(Insert(m_uSize, val)); }

    _Success_(return != FALSE)
    BOOL Insert(__in UINT_PTR uIndex, __in const T& val)
    {
        CListElement* pNew;

        if(nullptr == (pNew = new (std::nothrow) CListElement(val)))
        {
            SetLastError(ERROR_OUTOFMEMORY);
            return(FALSE);
        }

        if(!m_pHead)
        {
            m_pHead = m_pTail = pNew;
        }
        else if(uIndex >= m_uSize)
        {
            m_pTail->m_pNext = pNew;
            m_pTail = pNew;
        }
        else if(!uIndex)
        {
            pNew->m_pNext = m_pHead;
            m_pHead = pNew;
        }
        else
        {
            CListElement* pCurr = m_pHead;

            while(--uIndex)
                pCurr = pCurr->m_pNext;

            pNew->m_pNext = pCurr->m_pNext;
            pCurr->m_pNext = pNew;
        }

        ++m_uSize;
        return(TRUE);
    }

    _Success_(return != FALSE)
    BOOL Remove(__in UINT_PTR uIndex, __out_opt T* pval = nullptr)
    {
        CListElement* pRemove;

        if(uIndex >= m_uSize)
        {
            SetLastError(ERROR_INVALID_INDEX);
            return(FALSE);
        }

        if(!uIndex)
        {
            pRemove = m_pHead;
            m_pHead = pRemove->m_pNext;
        }
        else
        {
            CListElement* pPrev = m_pHead;

            while(--uIndex)
                pPrev = pPrev->m_pNext;

            pRemove = pPrev->m_pNext;

            if(nullptr == (pPrev->m_pNext = pRemove->m_pNext))
                m_pTail = pPrev;
        }

        if(pval)
            *pval = std::move(pRemove->m_val);

        --m_uSize;
        delete pRemove;
        return(TRUE);
    }

    _Success_(return != FALSE)
    BOOL Get(__in UINT_PTR uIndex, __out T* pval) const
    {
        if(uIndex >= m_uSize)
        {
            SetLastError(ERROR_INVALID_PARAMETER);
            return(FALSE);
        }
        else if(uIndex == (m_uSize - 1))
        {
            *pval = m_pTail->m_val;
        }
        else
        {
            const CListElement* pEntry = m_pHead;

            while(uIndex--)
                pEntry = pEntry->m_pNext;

            *pval = pEntry->m_val;
        }

        return(TRUE);
    }

    template<typename Predicate>
    void Enum(const Predicate&& predicate) const
    {
        for(CListElement* pElement = m_pHead; pElement; pElement = pElement->m_pNext)
            if(!predicate(pElement->m_val))
                break;
    }

    template<typename KEY>
    _Success_(return != FALSE)
    BOOL Find(__in const KEY& key, __out_opt T* pval = nullptr, __out_opt UINT_PTR* pidx = nullptr) const
    {
        UINT_PTR idx = 0;

        for(const CListElement* pElement = m_pHead; pElement; pElement = pElement->m_pNext)
        {
            if(Equals(key, pElement->m_val))
            {
                if(pidx)
                    *pidx = idx;

                if(pval)
                    *pval = pElement->m_val;

                return(TRUE);
            }
            ++idx;
        }

        return(FALSE);
    }

    template<typename KEY>
    _Success_(return != FALSE)
    BOOL FindAndRemove(__in const KEY& key, __out_opt T* pval = nullptr)
    {
        CListElement* pPrev = nullptr;

        for(CListElement* pElement = m_pHead; pElement; pElement = pElement->m_pNext)
        {
            if(Equals(key, pElement->m_val))
            {
                if(!pPrev)
                    m_pHead = pElement->m_pNext;
                else if(nullptr == (pPrev->m_pNext = pElement->m_pNext))
                    m_pTail = pPrev;

                if(pval)
                    *pval = std::move(pElement->m_val);

                --m_uSize;
                delete pElement;
                return(TRUE);
            }

            pPrev = pElement;
        }

        return(FALSE);
    }

    UINT_PTR Size() const { return(m_uSize); }

private:
    template<typename KEY, typename EQ> static inline bool Equals(__in const KEY& key, __in EQ* pval) { return(*pval == key); }
    template<typename KEY, typename EQ> static inline bool Equals(__in const KEY& key, __in const EQ& val) { return(val == key); }

    CList(const CList& other);
    CList& operator=(const CList& other);

protected:
    struct CListElement
    {
        explicit CListElement(const T& val) : m_pNext(nullptr), m_val(val) {}

        CListElement* m_pNext;
        T m_val;

        CListElement(const CListElement& other);
    };

    CListElement* m_pHead;
    CListElement* m_pTail;
    UINT_PTR      m_uSize;
};

#endif
