#ifndef CSMARTICE04_H
#define CSMARTICE04_H

enum StatusFlags
{
    TargetVoltagePresent = 0x01,
    DebugMode = 0x02
};

#include <pshpack1.h>

struct REPORT1
{
    BYTE    bReportId;
    BYTE    bChipId;
    BYTE    bVersion;
    BYTE    bStatus;
    WORD    wTargetVoltage;
};

#define CMDID_DEBUGMODE         0x01    /* param1 = 0xFFFF -> Enter, param1 = 0x0000 -> Leave                                                               */
#define CMDID_DEBUGCONFIG       0x02    /* param1 = 0x00xx -> xx = config data                                                                              */
#define CMDID_RESUME            0x03    /* ** no parameters **                                                                                              */
#define CMDID_HALT              0x04    /* ** no parameters **                                                                                              */
#define CMDID_CHIP_ERASE        0x05    /* ** no parameters                                                                                                 */
#define CMDID_DEBUG_INSTR       0x06    /* param1 low -> byte1, param1 high -> byte2, param2 low -> byte 3                                                  */
#define CMDID_STEP_REPLACE      0x07    /* param1 low -> byte1, param1 high -> byte2, param2 low -> byte 3, param2 high -> number of bytes used             */
#define CMDID_STEP_INSTR        0x08    /* ** no parameters **                                                                                              */
#define CMDID_SET_HW_BRKPNT     0x09    /* param1 -> Breakpoint address[0..15], param2 low -> Breakpoint address[16..23], param2 high -> Breakpoint Info    */

typedef struct
{
    BYTE    bReportId;
    BYTE    bCmd;
    WORD    param1;
    WORD    param2;
} REPORT2;

typedef struct
{
    BYTE            bReportId;
    BYTE            bSegment;
    unsigned long   uAddress;
    unsigned long   uLen;
    BYTE            blRead;
} REPORT3;

typedef struct
{
    BYTE    bReportId;
    BYTE    bLength;
    BYTE    bData[62];
} REPORT4;

typedef struct
{
    BYTE    bReportId;
    BYTE    bInDbgMode;
    BYTE    bConfig;
    BYTE    bStatus;
    BYTE    bResult;
} REPORT5;

typedef struct
{
    BYTE    bReportId;
    WORD    pc;
} REPORT6;

typedef struct
{
    BYTE    bReportId;
    BYTE    bLockBits[1];
    BYTE    bReserved[15];
} REPORT7;

typedef struct
{
    BYTE    bBank;
} REPORT8;

union IOREPORT
{
    REPORT1 rep1;
    REPORT2 rep2;
    REPORT3 rep3;
    REPORT4 rep4;
    REPORT5 rep5;
    REPORT6 rep6;
    REPORT7 rep7;
    REPORT8 rep8;
};

#include <poppack.h>

#define CSMARTICE04_STATUS_GETMEMORY    0x00000001
#define CSMARTICE04_STATUS_SETMEMORY    0x00000002
#define CSMARTICE04_STATUS_VERIFY       0x00000004

class CSmartICE04Devices;

class CSmartICE04 : public ISmartICE04, public IConnectionPointContainer, public IProvideClassInfo2, public ISupportErrorInfo
{
protected:
    inline IUnknown* GetUnknown() { return(static_cast<ISmartICE04*>(this));   }
    inline HRESULT RaiseCOMException(DISPID dispid, HRESULT hr) { return(::RaiseCOMException(this, dispid, hr));  }
    inline WORD getPID() const { return(m_wPID); }

    CSmartICE04(ITypeLib* pTlb, CSmartICE04Devices* pParent, HANDLE hDevice, LPCTSTR pszDeviceName, WORD wProductId, HRESULT* phr);

public:
    // IUnknown
    IFACEMETHOD(QueryInterface) (REFIID riid, void** ppvObject);
    IFACEMETHOD_(ULONG, AddRef) ();
    IFACEMETHOD_(ULONG, Release)();

    // IDispatch
    IFACEMETHOD(GetTypeInfoCount)(UINT* pctinfo);
    IFACEMETHOD(GetTypeInfo)(UINT iTInfo, LCID lcid, ITypeInfo** ppTInfo);
    IFACEMETHOD(GetIDsOfNames)(REFIID riid, LPOLESTR* rgszNames, UINT cNames, LCID lcid, DISPID* rgDispId);
    IFACEMETHOD(Invoke)(DISPID dispIdMember, REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS* pDispParams, VARIANT* pVarResult, EXCEPINFO* pExcepInfo, UINT* puArgErr);

    // ISmartICE04
    IFACEMETHOD(get_Parent)(IDispatch** ppParent);
    IFACEMETHOD(get_ProductId)(long* PID);
    IFACEMETHOD(get_SerialNumber)(BSTR* SerNum);
    IFACEMETHOD(get_Icon)(IPictureDisp** Picture);
    IFACEMETHOD(get_SmallIcon)(IPictureDisp** Picture);
    IFACEMETHOD(get_ChipId)(ChipId* cid);
    IFACEMETHOD(get_ChipVersion)(long* vers);
    IFACEMETHOD(get_TargetVoltage)(long* targetVoltage);
    IFACEMETHOD(EnterDebugMode)(VARIANT_BOOL enableDebug);
    IFACEMETHOD(get_IsInDebugMode)(VARIANT_BOOL* isInDbgMode);
    IFACEMETHOD(get_DebugConfig)(long* configFlags);
    IFACEMETHOD(put_DebugConfig)(long configFlags);
    IFACEMETHOD(ChipErase)();
    IFACEMETHOD(Halt)();
    IFACEMETHOD(Resume)();
    IFACEMETHOD(ReadStatus)(long* statusFlags);
    IFACEMETHOD(DebugInstr)(long cmd, long* result);
    IFACEMETHOD(StepReplace)(long cmd, long* result);
    IFACEMETHOD(StepInstr)(long* result);
    IFACEMETHOD(SetHwBreakpoint)(long brkpntNumber, VARIANT_BOOL enable, long address);
    IFACEMETHOD(GetPc)(long* pc);
    IFACEMETHOD(ReadLockBits)(long* lockbits);
    IFACEMETHOD(WriteLockBits)(long lockbits);
    IFACEMETHOD(GetMemory)(MemorySegment seg, long startAddress, long len, VARIANT* progressItf, SAFEARRAY** data);
    IFACEMETHOD(SetMemory)(MemorySegment seg, long startAddress, SAFEARRAY** data, VARIANT_BOOL verify, VARIANT* progressItf);
    IFACEMETHOD(DownloadFile)(BSTR filename, FileType ft, VARIANT_BOOL verify, VARIANT* progressItf);
    IFACEMETHOD(GetBm)(long* bm);

    // IConnectionPointContainer
    IFACEMETHOD(EnumConnectionPoints)(IEnumConnectionPoints** ppEnum);
    IFACEMETHOD(FindConnectionPoint)(REFIID riid, IConnectionPoint** ppCP);

    // IProvideClassInfo2
    IFACEMETHOD(GetClassInfo)(ITypeInfo** ppTI);
    IFACEMETHOD(GetGUID)(DWORD dwGuidKind, GUID* pGUID);

    // ISupportErrorInfo
    IFACEMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

    static NTSTATUS WINAPI HidP_InitializeReportForID(HIDP_REPORT_TYPE ReportType, UCHAR ReportID, PHIDP_PREPARSED_DATA PreparsedData, PCHAR Report, ULONG ReportLength);
    static HRESULT Create(__in ITypeLib* pTlb, __in CSmartICE04Devices* pParent, __in HANDLE hDevice, __in_z LPCTSTR pszDeviceName, __in WORD wProductId, __deref_out CSmartICE04** ppDev);

    void OnDetached();
    inline bool operator==(__in_z PCTSTR pszName) const { return(CSTR_EQUAL == CompareString(LOCALE_INVARIANT, NORM_IGNORECASE, pszName, -1, m_pszDeviceName, -1)); }
    inline bool operator==(__in BSTR SerialNumber) const { return(VARCMP_EQ == VarBstrCmp(SerialNumber, m_bstrSerialNumber, LOCALE_INVARIANT, NORM_IGNORECASE)); }

#pragma warning(suppress: 6031)
    inline void Refresh() { if(!(m_ObjectStatus & (CSMARTICE04_STATUS_GETMEMORY | CSMARTICE04_STATUS_SETMEMORY))) { IOREPORT report; ReadReport1(&report); }}

private:
    virtual ~CSmartICE04();
    CSmartICE04(const CSmartICE04& other);
    CSmartICE04& operator=(const CSmartICE04& other);

    __checkReturn HRESULT ReadReport1(__out IOREPORT* pReport);
    __checkReturn HRESULT ReadReport5(__out IOREPORT* pReport) const;
    __checkReturn HRESULT ReadReport6(__out IOREPORT* pReport) const;
    __checkReturn HRESULT ReadReport7(__out IOREPORT* pReport) const;
    __checkReturn HRESULT ReadReport8(__out IOREPORT* pReport) const;
    __checkReturn HRESULT WriteReport(__in const IOREPORT* pReport) const;
    static __checkReturn HRESULT ExtractProgressItf(__in const VARIANT* progressItf, __in CSmartICE04ProgressDummy* pDummy, __deref_out ISmartICE04Progress** ppProgressItf);

    CSmartICE04Devices* m_pParent;
    CDSmartICE04Events  m_DispEvents;
    CSmartICE04Events   m_Events;
    ITypeInfo*          m_pTypeInfo;
    HANDLE              m_hDevice;
    HANDLE              m_hMutex;
    LPTSTR              m_pszDeviceName;
    BYTE*               m_pFeatureReport;
    BYTE*               m_pInputReport;
    BSTR                m_bstrSerialNumber;
    BOOL                m_blDetached;
    BOOL                m_blInDebugMode;
    USHORT              m_usFeatureReportLength;
    USHORT              m_usInputReportLength;
    volatile ULONG      m_lRefCount;
    ULONG               m_ObjectStatus;
    REPORT1             m_report1;
    WORD                m_wPID;
    BYTE                m_bDebugConfig;
    BYTE bPadding[3]; // C4820
};

#endif
