#include "precomp.h"

ATOM CSmartICE04Devices::m_ClassAtom;

CSmartICE04Devices::CSmartICE04Devices(HRESULT* phr)
    : m_hDevNotify(nullptr), m_pTypeInfo(nullptr), m_pTlb(nullptr), m_hWnd(nullptr), m_lRefCount(1), m_blRefreshing(FALSE)
{
    DEV_BROADCAST_DEVICEINTERFACE   itf = {};
    WNDCLASSEX                      wc;
    HDEVINFO                        hDevInfo;
    DWORD                           dwIndex;

    ::LockServer();
    m_Events.SetContainer(this);
    m_DispEvents.SetContainer(this);

    wc.hInstance        = g_hInst;
    wc.lpszClassName    = TEXT("{FFE8C50A-7AE3-4F68-98E0-03679E7EF2A4}");

    if(!m_ClassAtom)
    {
        wc.cbSize           = sizeof(wc);
        wc.style            = 0;
        wc.lpfnWndProc      = WndProc;
        wc.cbClsExtra       = 0;
        wc.cbWndExtra       = 0;
        wc.hIcon            = nullptr;
        wc.hCursor          = nullptr;
        wc.hbrBackground    = nullptr;
        wc.lpszMenuName     = nullptr;
        wc.hIconSm          = nullptr;
        const ATOM atom = RegisterClassEx(&wc);

        if(atom)
        {
            m_ClassAtom = atom;
        }
        else
        {
            dwIndex = GetLastError();

            if(dwIndex != ERROR_CLASS_ALREADY_EXISTS)
            {
                *phr = HRESULT_FROM_WIN32(dwIndex);
                return;
            }
        }
    }

    if(nullptr == (m_hWnd = CreateWindowEx(0, wc.lpszClassName, nullptr, 0, 0, 0, 0, 0, HWND_MESSAGE, nullptr, wc.hInstance, this)))
    {
        *phr = HRESULT_FROM_WIN32(GetLastError());
        return;
    }
    else if(!SetTimer(m_hWnd, 1234, 2500, nullptr))
    {
        *phr = HRESULT_FROM_WIN32(GetLastError());
        return;
    }

    itf.dbcc_size       = sizeof(itf);
    itf.dbcc_devicetype = DBT_DEVTYP_DEVICEINTERFACE;
    HidD_GetHidGuid(&itf.dbcc_classguid);

    if(nullptr == (m_hDevNotify = RegisterDeviceNotification(m_hWnd, &itf, DEVICE_NOTIFY_WINDOW_HANDLE)))
    {
        *phr = HRESULT_FROM_WIN32(GetLastError());
        return;
    }

    if(SUCCEEDED(*phr = LoadRegTypeLib(LIBID_SmartICE04Lib, 1, 0, LOCALE_USER_DEFAULT, &m_pTlb)))
    {
        if(SUCCEEDED(*phr = m_pTlb->GetTypeInfoOfGuid(IID_ISmartICE04Devices, &m_pTypeInfo)))
        {
            if(INVALID_HANDLE_VALUE == (hDevInfo = SetupDiGetClassDevs(&itf.dbcc_classguid, nullptr, nullptr, DIGCF_DEVICEINTERFACE | DIGCF_PRESENT)))
            {
                dwIndex = GetLastError();
                *phr = HRESULT_FROM_SETUPAPI(dwIndex);
            }
            else
            {
                SP_DEVICE_INTERFACE_DATA spdid = {};
                spdid.cbSize = sizeof(spdid);

                for(dwIndex = 0; (SetupDiEnumDeviceInterfaces(hDevInfo, nullptr, &itf.dbcc_classguid, dwIndex, &spdid)); ++dwIndex)
                {
                    DWORD dwSize = 0;
                    SetupDiGetDeviceInterfaceDetail(hDevInfo, &spdid, nullptr, 0, &dwSize, nullptr);

                    if(dwSize >= sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA))
                    {
                        SP_DEVICE_INTERFACE_DETAIL_DATA* pspdidd = static_cast<PSP_DEVICE_INTERFACE_DETAIL_DATA>(calloc(dwSize, 1));

                        if(!pspdidd)
                            continue;

                        pspdidd->cbSize = sizeof(*pspdidd);

                        if(SetupDiGetDeviceInterfaceDetail(hDevInfo, &spdid, pspdidd, dwSize, &dwSize, nullptr))
                            Add(pspdidd->DevicePath);

                        free(pspdidd);
                    }
                }
                SetupDiDestroyDeviceInfoList(hDevInfo);
            }
        }
    }
}

CSmartICE04Devices::~CSmartICE04Devices()
{
    CSmartICE04* pDev;

    if(m_hDevNotify)
        UnregisterDeviceNotification(m_hDevNotify);

    if(m_hWnd)
    {
        KillTimer(m_hWnd, 1234);

        if(!DestroyWindow(m_hWnd))
            PostMessage(m_hWnd, WM_CLOSE, 0, 0);
    }

    while(m_Devices.Remove(0, &pDev))
        pDev->Release();

    if(m_pTypeInfo)
        m_pTypeInfo->Release();

    if(m_pTlb)
        m_pTlb->Release();

    ::UnlockServer();
}

BOOL
CSmartICE04Devices::Cleanup()
{
    BOOL ret = TRUE;

    if(m_ClassAtom)
    {
        ret = UnregisterClass(MAKEINTATOM(m_ClassAtom), g_hInst);
        m_ClassAtom = 0;
    }

    return(ret);
}

IFACEMETHODIMP
CSmartICE04Devices::QueryInterface(REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = nullptr;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_IDispatch))
            *ppvObject = static_cast<IDispatch*>(this);
        else if(IsEqualIID(riid, IID_ISmartICE04Devices))
            *ppvObject = static_cast<ISmartICE04Devices*>(this);
        else if(IsEqualIID(riid, IID_IConnectionPointContainer))
            *ppvObject = static_cast<IConnectionPointContainer*>(this);
        else if(IsEqualIID(riid, IID_IProvideClassInfo))
            *ppvObject = static_cast<IProvideClassInfo*>(this);
        else if(IsEqualIID(riid, IID_IProvideClassInfo2))
            *ppvObject = static_cast<IProvideClassInfo2*>(this);
        else if(IsEqualIID(riid, IID_ISupportErrorInfo))
            *ppvObject = static_cast<ISupportErrorInfo*>(this);

        if(*ppvObject)
        {
            static_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

IFACEMETHODIMP_(ULONG)
CSmartICE04Devices::AddRef()
{
    return(InterlockedIncrement(&m_lRefCount));
}

IFACEMETHODIMP_(ULONG)
CSmartICE04Devices::Release()
{
    const ULONG ulResult = InterlockedDecrement(&m_lRefCount);

    if(!ulResult)
        delete this;

    return(ulResult);
}

IFACEMETHODIMP
CSmartICE04Devices::GetTypeInfoCount(UINT* pctinfo)
{
    if(IsBadWritePtrT(pctinfo))
        return(E_POINTER);

    *pctinfo = 1;
    return(S_OK);
}

IFACEMETHODIMP
CSmartICE04Devices::GetTypeInfo(UINT iTInfo, LCID, ITypeInfo** ppTInfo)
{
    if(IsBadWritePtrT(ppTInfo))
        return(E_POINTER);

    if(iTInfo)
    {
        *ppTInfo = nullptr;
        return(DISP_E_BADINDEX);
    }

    *ppTInfo = m_pTypeInfo;
    (*ppTInfo)->AddRef();
    return(S_OK);
}

IFACEMETHODIMP
CSmartICE04Devices::GetIDsOfNames(REFIID, LPOLESTR* rgszNames, UINT cNames, LCID, DISPID* rgDispId)
{
    return(m_pTypeInfo->GetIDsOfNames(rgszNames, cNames, rgDispId));
}

IFACEMETHODIMP
CSmartICE04Devices::Invoke(DISPID dispIdMember, REFIID, LCID lcid, WORD wFlags, DISPPARAMS* pDispParams, VARIANT* pVarResult, EXCEPINFO* pExcepInfo, UINT* puArgErr)
{
    CThreadLocale loc(lcid);
    UINT uArgErr;
    return(m_pTypeInfo->Invoke(static_cast<ISmartICE04Devices*>(this), dispIdMember, wFlags, pDispParams, pVarResult, pExcepInfo, (puArgErr) ? puArgErr : &uArgErr));
}

IFACEMETHODIMP
CSmartICE04Devices::get_Item(const VARIANT varIndex, VARIANT* pVal)
{
    CSmartICE04*    pDev = nullptr;
    HRESULT         hr;
    VARIANT         var;

    if(IsBadWritePtrT(pVal))
        return(RaiseCOMException(DISPID_SMARTICE04_LISTITEM, E_POINTER));

    VariantInit(pVal);

    if(V_VT(&varIndex) == VT_BSTR)
    {
        if(!SysStringLen(V_BSTR(&varIndex)))
            return(RaiseCOMException(DISPID_SMARTICE04_LISTITEM, E_INVALIDARG));

        if(m_Devices.Find(V_BSTR(&varIndex), &pDev))
        {
            if(SUCCEEDED(hr = pDev->QueryInterface(IID_IDispatch, reinterpret_cast<void**>(&V_DISPATCH(pVal)))))
            {
                V_VT(pVal) = VT_DISPATCH;
                return(S_OK);
            }

            return(RaiseCOMException(DISPID_SMARTICE04_LISTITEM, hr));
        }
    }

    VariantInit(&var);

    if(SUCCEEDED(hr = VariantChangeTypeEx(&var, &varIndex, CThreadLocale::getLCID(), 0, VT_I4)))
    {
        if(!m_Devices.Get(static_cast<UINT_PTR>(V_I4(&var)), &pDev))
        {
            hr = E_INVALIDARG;
        }
        else if(SUCCEEDED(hr = pDev->QueryInterface(IID_IDispatch, reinterpret_cast<void**>(&V_DISPATCH(pVal)))))
        {
            V_VT(pVal) = VT_DISPATCH;
        }
    }
    else if(V_VT(&varIndex) == VT_BSTR)
    {
        hr = E_INVALIDARG;
    }

    VariantClear(&var);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_LISTITEM, hr));
}

IFACEMETHODIMP
CSmartICE04Devices::get_Count(long* pVal)
{
    const HRESULT hr = getCount(pVal);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_LISTCOUNT, hr));
}

IFACEMETHODIMP
CSmartICE04Devices::get_length(long* pVal)
{
    const HRESULT hr = getCount(pVal);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_COLLCOUNT, hr));
}

IFACEMETHODIMP
CSmartICE04Devices::get__NewEnum(IUnknown** ppEnum)
{
    CEnumVARIANT* pEnum;

    if(IsBadWritePtrT(ppEnum))
        return(RaiseCOMException(DISPID_SMARTICE04__NEWENUM, E_POINTER));

    *ppEnum = nullptr;

    if(nullptr == (pEnum = new (std::nothrow) CEnumVARIANT(this)))
        return(RaiseCOMException(DISPID_SMARTICE04__NEWENUM, E_OUTOFMEMORY));

    const HRESULT hr = pEnum->QueryInterface(IID_IUnknown, reinterpret_cast<void**>(ppEnum));
    pEnum->Release();
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04__NEWENUM, hr));
}

IFACEMETHODIMP
CSmartICE04Devices::EnumConnectionPoints(IEnumConnectionPoints** ppEnum)
{
    CEnumConnectionPoints*  pEnum;
    HRESULT                 hr;

    if(IsBadWritePtrT(ppEnum))
        return(E_POINTER);

    *ppEnum = nullptr;

    if(nullptr == (pEnum = new (std::nothrow) CEnumConnectionPoints))
        return(E_OUTOFMEMORY);

    if(SUCCEEDED(hr = pEnum->Add(m_DispEvents.GetUnknown())))
        if(SUCCEEDED(hr = pEnum->Add(m_Events.GetUnknown())))
            hr = pEnum->QueryInterface(IID_IEnumConnectionPoints, reinterpret_cast<void**>(ppEnum));

    pEnum->Release();
    return(hr);
}

IFACEMETHODIMP
CSmartICE04Devices::FindConnectionPoint(REFIID riid, IConnectionPoint** ppCP)
{
    __try
    {
        *ppCP = nullptr;

        if(IsEqualIID(riid, IID_ISmartICE04DevicesEvents))
            return(m_Events.QueryInterface(IID_IConnectionPoint, reinterpret_cast<void**>(ppCP)));

        if(IsEqualIID(riid, DIID__DISmartICE04DevicesEvents))
            return(m_DispEvents.QueryInterface(IID_IConnectionPoint, reinterpret_cast<void**>(ppCP)));
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_POINTER);
    }

    return(CONNECT_E_NOCONNECTION);
}

IFACEMETHODIMP
CSmartICE04Devices::GetClassInfo(ITypeInfo** ppTI)
{
    if(IsBadWritePtrT(ppTI))
        return(E_POINTER);

    return(m_pTlb->GetTypeInfoOfGuid(CLSID_SmartICE04Devices, ppTI));
}

IFACEMETHODIMP
CSmartICE04Devices::GetGUID(DWORD dwGuidKind, GUID* pGUID)
{
    if(IsBadWritePtrT(pGUID))
        return(E_POINTER);

    if(dwGuidKind != GUIDKIND_DEFAULT_SOURCE_DISP_IID)
    {
        *pGUID = GUID_NULL;
        return(E_INVALIDARG);
    }

    *pGUID = DIID__DISmartICE04DevicesEvents;
    return(S_OK);
}

IFACEMETHODIMP
CSmartICE04Devices::InterfaceSupportsErrorInfo(REFIID riid)
{
    __try
    {
        if(IsEqualIID(riid, IID_ISmartICE04Devices))
            return(S_OK);
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(S_FALSE);
}

HRESULT
CSmartICE04Devices::getCount(long* pVal) const
{
    if(IsBadWritePtrT(pVal))
        return(E_POINTER);

    *pVal = static_cast<long>(m_Devices.Size());
    return(S_OK);
}

void
CSmartICE04Devices::Refresh()
{
    if(!m_blRefreshing)
    {
        m_blRefreshing = TRUE;

        m_Devices.Enum([](CSmartICE04* pDev) -> bool
        {
            pDev->Refresh();
            return(true);
        });

        m_blRefreshing = FALSE;
    }
}

HRESULT
CSmartICE04Devices::Add(LPCTSTR pszName)
{
    HIDD_ATTRIBUTES attr = {};
    CSmartICE04*    pDev;
    ITypeLib*       pTlb = nullptr;
    HRESULT         hr;
    HANDLE          hDevice;
    UINT            x;

    if(m_Devices.Find(pszName))
        return(S_OK);

    if(INVALID_HANDLE_VALUE == (hDevice = CreateFile(pszName, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, nullptr, OPEN_EXISTING, 0, nullptr)))
        return(HRESULT_FROM_WIN32(GetLastError()));

    attr.Size = sizeof(attr);

    if(!HidD_GetAttributes(hDevice, &attr))
    {
        hr = HRESULT_FROM_WIN32(GetLastError());
    }
    else if((attr.VendorID != 0x14AB) || (attr.ProductID != 0xCC11))
    {
        hr = E_INVALIDARG;
    }
    else
    {
        if(SUCCEEDED(hr = m_pTypeInfo->GetContainingTypeLib(&pTlb, &x)))
        {
            if(SUCCEEDED(hr = CSmartICE04::Create(pTlb, this, hDevice, pszName, attr.ProductID, &pDev)))
            {
                hDevice = INVALID_HANDLE_VALUE; // pDev took ownership

                if(!m_Devices.Add(pDev))
                {
                    hr = HRESULT_FROM_WIN32(GetLastError());
                    pDev->Release();
                }
                else
                {
                    m_Events.FireEvent(DISPID_SMARTICE04_ONDEVICEATTACHED, pDev);
                    m_DispEvents.FireEvent(DISPID_SMARTICE04_ONDEVICEATTACHED, pDev);
                }
            }
            pTlb->Release();
        }
    }

    if(INVALID_HANDLE_VALUE != hDevice)
         CloseHandle(hDevice);

    return(hr);
}

HRESULT
CSmartICE04Devices::Add(const DEV_BROADCAST_DEVICEINTERFACE* pItf)
{
    GUID guid;
    HidD_GetHidGuid(&guid);

    if(!IsEqualGUID(guid, pItf->dbcc_classguid))
        return(E_INVALIDARG);

#ifdef UNICODE
    if(!IsWindowUnicode(m_hWnd))
    {
        BSTR bstr;
        HRESULT hr = BSTRFromStringA(reinterpret_cast<const DEV_BROADCAST_DEVICEINTERFACE_A*>(pItf)->dbcc_name, &bstr);

        if(SUCCEEDED(hr))
        {
            hr = Add(bstr);
            SysFreeString(bstr);
        }

        return(hr);
    }
#endif

    return(Add(pItf->dbcc_name));
}

HRESULT
CSmartICE04Devices::Remove(LPCTSTR pszName)
{
    CSmartICE04* pDev;

    if(m_Devices.FindAndRemove(pszName, &pDev))
    {
        m_Events.FireEvent(DISPID_SMARTICE04_ONDEVICEDETACHED, pDev);
        m_DispEvents.FireEvent(DISPID_SMARTICE04_ONDEVICEDETACHED, pDev);
        pDev->OnDetached();
        pDev->Release();
        return(S_OK);
    }

    return(E_INVALIDARG);
}

HRESULT
CSmartICE04Devices::Remove(const DEV_BROADCAST_DEVICEINTERFACE* pItf)
{
#ifdef UNICODE
    if(!IsWindowUnicode(m_hWnd))
    {
        BSTR bstr;
        HRESULT hr = BSTRFromStringA(reinterpret_cast<const DEV_BROADCAST_DEVICEINTERFACE_A*>(pItf)->dbcc_name, &bstr);

        if(SUCCEEDED(hr))
        {
            hr = Remove(bstr);
            SysFreeString(bstr);
        }

        return(hr);
    }
#endif

    return(Remove(pItf->dbcc_name));
}

LRESULT CALLBACK
CSmartICE04Devices::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
        case WM_NCCREATE:
            SetWindowLongPtr(hWnd, GWLP_USERDATA, reinterpret_cast<LONG_PTR>(reinterpret_cast<CREATESTRUCT*>(lParam)->lpCreateParams));
            break;
        case WM_TIMER:
            if(wParam == 1234)
                reinterpret_cast<CSmartICE04Devices*>(GetWindowLongPtr(hWnd, GWLP_USERDATA))->Refresh();

            return(0);
        case WM_DEVICECHANGE:
            if(wParam == DBT_DEVICEARRIVAL)
            {
                if(reinterpret_cast<DEV_BROADCAST_HDR*>(lParam)->dbch_devicetype == DBT_DEVTYP_DEVICEINTERFACE)
                    reinterpret_cast<CSmartICE04Devices*>(GetWindowLongPtr(hWnd, GWLP_USERDATA))->Add(reinterpret_cast<DEV_BROADCAST_DEVICEINTERFACE*>(lParam));
            }
            else if(wParam == DBT_DEVICEREMOVECOMPLETE)
            {
                if(reinterpret_cast<DEV_BROADCAST_HDR*>(lParam)->dbch_devicetype == DBT_DEVTYP_DEVICEINTERFACE)
                    reinterpret_cast<CSmartICE04Devices*>(GetWindowLongPtr(hWnd, GWLP_USERDATA))->Remove(reinterpret_cast<DEV_BROADCAST_DEVICEINTERFACE*>(lParam));
            }
            return(TRUE);
        default:
            break;
    }

    return(DefWindowProc(hWnd, uMsg, wParam, lParam));
}
