#include "precomp.h"

IFACEMETHODIMP
CSmartICE04DevicesEventsSink::QueryInterface(REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = nullptr;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_ISmartICE04DevicesEvents))
            *ppvObject = static_cast<ISmartICE04DevicesEvents*>(this);

        if(*ppvObject)
        {
            static_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

IFACEMETHODIMP_(ULONG)
CSmartICE04DevicesEventsSink::AddRef()
{
    return(2);
}

IFACEMETHODIMP_(ULONG)
CSmartICE04DevicesEventsSink::Release()
{
    return(1);
}

IFACEMETHODIMP
CSmartICE04DevicesEventsSink::OnDeviceAttached(IDispatch*, IDispatch* device)
{
    ISmartICE04* pDevice = nullptr;
    if(SUCCEEDED(device->QueryInterface(IID_ISmartICE04, reinterpret_cast<void**>(&pDevice))))
    {
        AddDevice(pDevice);
        pDevice->Release();
    }
    return(S_OK);
}

IFACEMETHODIMP
CSmartICE04DevicesEventsSink::OnDeviceDetached(IDispatch*, IDispatch* device)
{
    ISmartICE04* pDevice = nullptr;
    if(SUCCEEDED(device->QueryInterface(IID_ISmartICE04, reinterpret_cast<void**>(&pDevice))))
    {
        RemoveDevice(pDevice);
        pDevice->Release();
    }
    return(S_OK);
}

HRESULT
CSmartICE04DevicesEventsSink::Connect()
{
    ISmartICE04Devices* pDevices;
    HRESULT             hr = S_OK;

    if(nullptr == (pDevices = new (std::nothrow) CSmartICE04Devices(&hr)))
        return(E_OUTOFMEMORY);

    if(SUCCEEDED(hr))
        hr = ConnectTo(pDevices, IID_ISmartICE04DevicesEvents);

    pDevices->Release();
    return(hr);
}

HRESULT
CSmartICE04DevicesEventsSink::EnumDevices()
{
    IConnectionPointContainer*  pCPC = nullptr;
    ISmartICE04Devices*         pDevices = nullptr;
    VARIANT                     varIndex, varDevice;
    HRESULT                     hr;

    if(!m_pCP)
        return(E_FAIL);

    if(FAILED(hr = m_pCP->GetConnectionPointContainer(&pCPC)))
        return(hr);

    if(SUCCEEDED(hr = pCPC->QueryInterface(IID_ISmartICE04Devices, reinterpret_cast<void**>(&pDevices))))
    {
        VariantInit(&varIndex);
        VariantInit(&varDevice);
        V_VT(&varIndex) = VT_I4;

        for(V_I4(&varIndex) = 0; SUCCEEDED(pDevices->get_Item(varIndex, &varDevice)); ++V_I4(&varIndex))
        {
            if(SUCCEEDED(VariantChangeTypeEx(&varDevice, &varDevice, CThreadLocale::getLCID(), 0, VT_DISPATCH)))
                if(V_DISPATCH(&varDevice))
                    OnDeviceAttached(pDevices, V_DISPATCH(&varDevice));

            VariantClear(&varDevice);
        }

        pDevices->Release();
    }

    pCPC->Release();
    return(hr);
}
