#include "precomp.h"

CSmartICE04ProgressDialog::CSmartICE04ProgressDialog(HRESULT* phr)
    : m_pTypeInfo(nullptr), m_pTlb(nullptr), m_bstrTitle(SysAllocString(L"SmartICE04")), m_bstrLine1(nullptr), m_bstrLine2(nullptr), m_lRefCount(1), m_lcid(CThreadLocale::getLCID())
{
    ::LockServer();

    if(!m_bstrTitle)
        *phr = E_OUTOFMEMORY;
    else if(SUCCEEDED(*phr = LoadRegTypeLib(LIBID_SmartICE04Lib, 1, 0, LOCALE_USER_DEFAULT, &m_pTlb)))
        *phr = m_pTlb->GetTypeInfoOfGuid(IID_ISmartICE04ProgressDialog, &m_pTypeInfo);
}

CSmartICE04ProgressDialog::~CSmartICE04ProgressDialog()
{
    SysFreeString(m_bstrLine2);
    SysFreeString(m_bstrLine1);
    SysFreeString(m_bstrTitle);

    if(m_pTypeInfo)
        m_pTypeInfo->Release();

    if(m_pTlb)
        m_pTlb->Release();

    ::UnlockServer();
}

IFACEMETHODIMP
CSmartICE04ProgressDialog::QueryInterface(REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = nullptr;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_IDispatch))
            *ppvObject = static_cast<IDispatch*>(this);
        else if(IsEqualIID(riid, IID_ISmartICE04ProgressDialog))
            *ppvObject = static_cast<ISmartICE04ProgressDialog*>(this);
        else if(IsEqualIID(riid, IID_IProvideClassInfo))
            *ppvObject = static_cast<IProvideClassInfo*>(this);
        else if(IsEqualIID(riid, IID_ISupportErrorInfo))
            *ppvObject = static_cast<ISupportErrorInfo*>(this);

        if(*ppvObject)
        {
            static_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

IFACEMETHODIMP_(ULONG)
CSmartICE04ProgressDialog::AddRef()
{
    return(InterlockedIncrement(&m_lRefCount));
}

IFACEMETHODIMP_(ULONG)
CSmartICE04ProgressDialog::Release()
{
    const ULONG ulResult = InterlockedDecrement(&m_lRefCount);

    if(!ulResult)
        delete this;

    return(ulResult);
}

IFACEMETHODIMP
CSmartICE04ProgressDialog::GetTypeInfoCount(UINT* pctinfo)
{
    if(IsBadWritePtrT(pctinfo))
        return(E_POINTER);

    *pctinfo = 1;
    return(S_OK);
}

IFACEMETHODIMP
CSmartICE04ProgressDialog::GetTypeInfo(UINT iTInfo, LCID, ITypeInfo** ppTInfo)
{
    if(IsBadWritePtrT(ppTInfo))
        return(E_POINTER);

    if(iTInfo)
    {
        *ppTInfo = nullptr;
        return(DISP_E_BADINDEX);
    }

    *ppTInfo = m_pTypeInfo;
    (*ppTInfo)->AddRef();
    return(S_OK);
}

IFACEMETHODIMP
CSmartICE04ProgressDialog::GetIDsOfNames(REFIID, LPOLESTR* rgszNames, UINT cNames, LCID, DISPID* rgDispId)
{
    return(m_pTypeInfo->GetIDsOfNames(rgszNames, cNames, rgDispId));
}

IFACEMETHODIMP
CSmartICE04ProgressDialog::Invoke(DISPID dispIdMember, REFIID, LCID lcid, WORD wFlags, DISPPARAMS* pDispParams, VARIANT* pVarResult, EXCEPINFO* pExcepInfo, UINT* puArgErr)
{
    CThreadLocale loc(lcid);
    UINT uArgErr;
    return(m_pTypeInfo->Invoke(static_cast<ISmartICE04ProgressDialog*>(this), dispIdMember, wFlags, pDispParams, pVarResult, pExcepInfo, (puArgErr) ? puArgErr : &uArgErr));
}

IFACEMETHODIMP
CSmartICE04ProgressDialog::get_Title(BSTR *Title)
{
    const HRESULT hr = CopyBSTR(Title, m_bstrTitle);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_TITLE, hr));
}

IFACEMETHODIMP
CSmartICE04ProgressDialog::put_Title(BSTR Title)
{
    const HRESULT hr = ReplaceBSTR(&m_bstrTitle, Title);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_TITLE, hr));
}

IFACEMETHODIMP
CSmartICE04ProgressDialog::get_Line1(BSTR *line)
{
    const HRESULT hr = CopyBSTR(line, m_bstrLine1);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_LINE1, hr));
}

IFACEMETHODIMP
CSmartICE04ProgressDialog::put_Line1(BSTR line)
{
    const HRESULT hr = ReplaceBSTR(&m_bstrLine1, line);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_LINE1, hr));
}

IFACEMETHODIMP
CSmartICE04ProgressDialog::get_Line2(BSTR *line)
{
    const HRESULT hr = CopyBSTR(line, m_bstrLine2);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_LINE2, hr));
}

IFACEMETHODIMP
CSmartICE04ProgressDialog::put_Line2(BSTR line)
{
    const HRESULT hr = ReplaceBSTR(&m_bstrLine2, line);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_LINE2, hr));
}

IFACEMETHODIMP
CSmartICE04ProgressDialog::get_LocaleID(LCID *lcid)
{
    if(IsBadWritePtrT(lcid))
        return(RaiseCOMException(DISPID_SMARTICE04_LOCALEID, E_POINTER));

    *lcid = m_lcid;
    return(S_OK);
}

IFACEMETHODIMP
CSmartICE04ProgressDialog::put_LocaleID(LCID lcid)
{
    CThreadLocale loc(lcid);
    m_lcid = CThreadLocale::getLCID();
    return(S_OK);
}

IFACEMETHODIMP
CSmartICE04ProgressDialog::Create(OAHWND hWndParent, IDispatch** ppProgress)
{
    CSmartICE04Progress*    pProgress;
    HRESULT                 hr = S_OK;

    if(IsBadWritePtrT(ppProgress))
        return(RaiseCOMException(DISPID_SMARTICE04_CREATE, E_POINTER));

    *ppProgress = nullptr;

    if(nullptr == (pProgress = new (std::nothrow) CSmartICE04Progress(m_pTlb, this, reinterpret_cast<HWND>(hWndParent), &hr)))
    {
        hr = E_OUTOFMEMORY;
    }
    else
    {
        if(SUCCEEDED(hr))
            hr = pProgress->QueryInterface(IID_IDispatch, reinterpret_cast<void**>(ppProgress));

        pProgress->Release();
    }

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_CREATE, hr));
}

IFACEMETHODIMP
CSmartICE04ProgressDialog::GetClassInfo(ITypeInfo** ppTI)
{
    if(IsBadWritePtrT(ppTI))
        return(E_POINTER);

    return(m_pTlb->GetTypeInfoOfGuid(CLSID_SmartICE04ProgressDialog, ppTI));
}

IFACEMETHODIMP
CSmartICE04ProgressDialog::InterfaceSupportsErrorInfo(REFIID riid)
{
    __try
    {
        if(IsEqualIID(riid, IID_ISmartICE04ProgressDialog))
            return(S_OK);
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(S_FALSE);
}

HRESULT
CSmartICE04ProgressDialog::ReplaceBSTR(BSTR* pbstrOld, BSTR bstrNew)
{
    BSTR bstr;
    const HRESULT hr = CopyBSTR(&bstr, bstrNew);

    if(SUCCEEDED(hr))
    {
        SysFreeString(*pbstrOld);
        *pbstrOld = bstr;
    }

    return(hr);
}

HRESULT
CSmartICE04ProgressDialog::CopyBSTR(BSTR* pbstrDest, BSTR bstrSrc)
{
    if(IsBadWritePtrT(pbstrDest))
        return(E_POINTER);

    if(nullptr == (*pbstrDest = SysAllocStringLen(bstrSrc, SysStringLen(bstrSrc))))
        return((bstrSrc) ? E_OUTOFMEMORY : S_OK);

    return(S_OK);
}
