#include "precomp.h"

CTargetSetup::CTargetSetup()
    : m_hWnd(nullptr)
{
    m_Sink.SetParent(this);
    m_Sink.Connect();
}

HRESULT
CTargetSetup::DoModal(HWND hWndParent)
{
    CThreadDpiAwarenessContext dpiAwarness(DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE_V2);
    CTargetSetup dlg;
    CActCtx ctx(IDR_MANIFEST);
    const INT_PTR n = DialogBoxParam(g_hInst, MAKEINTRESOURCE(IDD_TARGETSETUP), hWndParent, DlgProc, reinterpret_cast<LPARAM>(&dlg));
    return((n == -1) ? HRESULT_FROM_WIN32(GetLastError()) : ((n == IDOK) ? S_OK : S_FALSE));
}

INT_PTR
CTargetSetup::OnWmInitDialog(HWND hWnd, WPARAM, LPARAM)
{
    TCHAR szCueBanner[64];
    int idx;

    m_hWnd = hWnd;
    CheckDlgButton(hWnd, IDC_CACHE_DATA, static_cast<UINT>((MonConf.Opt & CACHE_DATA) ? BST_CHECKED : BST_UNCHECKED));
    CheckDlgButton(hWnd, IDC_CACHE_XDATA, static_cast<UINT>((MonConf.Opt & CACHE_XDATA) ? BST_CHECKED : BST_UNCHECKED));
    CheckDlgButton(hWnd, IDC_CACHE_CODE, static_cast<UINT>((MonConf.Opt & CACHE_CODE) ? BST_CHECKED : BST_UNCHECKED));
    CheckRadioButton(hWnd, IDC_DA_USEFIRST, IDC_DA_SPECIFIC, ((MonConf.Opt & DA_SPECIFIC) ? IDC_DA_SPECIFIC : ((MonConf.Opt & DA_ASKUSER) ? IDC_DA_ASKUSER : IDC_DA_USEFIRST)));
    SetDlgItemTextA(hWnd, IDC_DA, MonConf.idDevice);
    EnableDlgItem(hWnd, IDC_DA, (BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_DA_SPECIFIC)));
    LoadString(g_hInst, IDS_NODEVICEDETECTED, szCueBanner, _countof(szCueBanner));
    SendDlgItemMessage(hWnd, IDC_DA, CB_SETCUEBANNER, 0, reinterpret_cast<LPARAM>(szCueBanner));
    m_Sink.EnumDevices();

    if(0 <= (idx = static_cast<int>(SendDlgItemMessageA(hWnd, IDC_DA, CB_FINDSTRINGEXACT, 0, reinterpret_cast<LPARAM>(MonConf.idDevice)))))
        SendDlgItemMessage(hWnd, IDC_DA, CB_SETCURSEL, static_cast<WPARAM>(idx), 0);

    return(TRUE);
}

INT_PTR
CTargetSetup::OnWmCommand(HWND hWnd, WPARAM wParam, LPARAM)
{
    switch(LOWORD(wParam))
    {
        case IDC_DA_USEFIRST:
        case IDC_DA_ASKUSER:
        case IDC_DA_SPECIFIC:
            if(HIWORD(wParam) == BN_CLICKED)
                EnableDlgItem(hWnd, IDC_DA, (BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_DA_SPECIFIC)));

            break;
        case IDCANCEL:
            OnWmInitDialog(hWnd, 0, reinterpret_cast<LPARAM>(this));
        case IDOK:
            MonConf.Opt &= ~(CACHE_DATA | CACHE_XDATA | CACHE_CODE | DA_ASKUSER | DA_SPECIFIC);

            if(BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_CACHE_DATA))
                MonConf.Opt |= CACHE_DATA;
            if(BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_CACHE_XDATA))
                MonConf.Opt |= CACHE_XDATA;
            if(BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_CACHE_CODE))
                MonConf.Opt |= CACHE_CODE;
            if(BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_DA_ASKUSER))
                MonConf.Opt |= DA_ASKUSER;
            if(BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_DA_SPECIFIC))
                MonConf.Opt |= DA_SPECIFIC;

            GetDlgItemTextA(hWnd, IDC_DA, MonConf.idDevice, _countof(MonConf.idDevice));
            EndDialog(hWnd, LOWORD(wParam));
            break;
        default:
            return(FALSE);
    }

    return(TRUE);
}

INT_PTR CALLBACK
CTargetSetup::DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    static const DLG_MSGMAP<CTargetSetup> sc_map[] =
    {
        { WM_COMMAND,   &CTargetSetup::OnWmCommand  }
    };

    for(uintptr_t n = 0; n < _countof(sc_map); ++n)
    {
        if(sc_map[n].uMsg == uMsg)
        {
            CTargetSetup* pThis = reinterpret_cast<CTargetSetup*>(GetWindowLongPtr(hWnd, DWLP_USER));

            if(pThis)
                return((pThis->*sc_map[n].pfn)(hWnd, wParam, lParam));
        }
    }

    if(uMsg == WM_INITDIALOG)
    {
        SetWindowLongPtr(hWnd, DWLP_USER, lParam);
        return(reinterpret_cast<CTargetSetup*>(lParam)->OnWmInitDialog(hWnd, wParam, lParam));
    }

    return(FALSE);
}

void
CTargetSetup::AddDevice(ISmartICE04* pDevice)
{
    BSTR sernum = nullptr;

    if(FAILED(pDevice->get_SerialNumber(&sernum)))
        return;

    if(0 > SendDlgItemMessage(m_hWnd, IDC_DA, CB_FINDSTRINGEXACT, static_cast<WPARAM>(-1), reinterpret_cast<LPARAM>(sernum)))
    {
        const LRESULT item = SendDlgItemMessageW(m_hWnd, IDC_DA, CB_ADDSTRING, 0, reinterpret_cast<LPARAM>(sernum));

        if(item >= 0)
        {
            if(!item)
            {
                TCHAR szCueBanner[64];
                LoadString(g_hInst, IDS_NODEVICESELECTED, szCueBanner, _countof(szCueBanner));
                SendDlgItemMessage(m_hWnd, IDC_DA, CB_SETCUEBANNER, 0, reinterpret_cast<LPARAM>(szCueBanner));
            }

            if(CB_ERR == SendDlgItemMessage(m_hWnd, IDC_DA, CB_GETCURSEL, 0, 0))
                SendDlgItemMessage(m_hWnd, IDC_DA, CB_SETCURSEL, static_cast<WPARAM>(item), 0);
        }
    }

    SysFreeString(sernum);
}

void
CTargetSetup::RemoveDevice(ISmartICE04* pDevice)
{
    LRESULT item;
    BSTR sernum = nullptr;

    if(FAILED(pDevice->get_SerialNumber(&sernum)))
        return;

    if(CB_ERR != (item = SendDlgItemMessageW(m_hWnd, IDC_DA, CB_FINDSTRINGEXACT, static_cast<WPARAM>(-1), reinterpret_cast<LPARAM>(sernum))))
    {
        TCHAR szCueBanner[64];

        if(item == SendDlgItemMessage(m_hWnd, IDC_DA, CB_GETCURSEL, 0, 0))
            InvalidateRect(GetDlgItem(m_hWnd, IDC_DA), nullptr, FALSE);

        SendDlgItemMessageW(m_hWnd, IDC_DA, CB_DELETESTRING, static_cast<WPARAM>(item), 0);
        LoadString(g_hInst, static_cast<UINT>((SendDlgItemMessage(m_hWnd, IDC_DA, CB_GETCOUNT, 0, 0) > 1) ? IDS_NODEVICESELECTED : IDS_NODEVICEDETECTED), szCueBanner, _countof(szCueBanner));
        SendDlgItemMessage(m_hWnd, IDC_DA, CB_SETCUEBANNER, 0, reinterpret_cast<LPARAM>(szCueBanner));
    }

    SysFreeString(sernum);
}
