#include "precomp.h"

HRESULT
CSiUtil::Download(char* pDownloadFile, BOOL bDeviceErase)
{
    const HRESULT hr = Connect();

    if(FAILED(hr))
        return(hr);

    return ::Download(pDownloadFile, bDeviceErase, FALSE, FALSE, -1, FALSE, FALSE);
}

HRESULT
CSiUtil::GetCodeMemory(BYTE* ptrMem, DWORD wStartAddress, unsigned int nLength)
{
    const HRESULT hr = Connect();

    if(FAILED(hr))
        return(hr);

    return ::GetCodeMemory(ptrMem, wStartAddress, nLength);
}

HRESULT
CSiUtil::SetCodeMemory(BYTE* ptrMem, DWORD wStartAddress, unsigned int nLength)
{
    const HRESULT hr = Connect();

    if(FAILED(hr))
        return(hr);

    return ::SetCodeMemory(ptrMem, wStartAddress, nLength, FALSE);
}

HRESULT
CSiUtil::SetTargetGo()
{
    const HRESULT hr = Connect();

    if(FAILED(hr))
        return(hr);

    return ::SetTargetGo();
}

HRESULT
CSiUtil::SetAppName(const char* pName)
{
    return ::SetAppName(pName);
}

CSiUtilEC2::CSiUtilEC2(int nCommPort)
    : m_nCommPort(nCommPort)
{
}

CSiUtilEC2::~CSiUtilEC2()
{
    ::Disconnect(m_nCommPort);
}

HRESULT
CSiUtilEC2::Connect()
{
    if(::Connected())
        return S_OK;

    const HRESULT hr = ::Connect(m_nCommPort, FALSE, 1, 0);

    if(FAILED(hr))
        MessageBoxA(nullptr, ::GetErrorMsg(hr), nullptr, MB_OK);

    return(hr);
}

HRESULT
CSiUtilEC2::Disconnect()
{
    if(!::Connected())
        return S_OK;

    return ::Disconnect(m_nCommPort);
}

bool
CSiUtilEC2::operator==(const char* sernum)
{
    if(!IS_INTRESOURCE(sernum))
        return false;

    return PtrToInt(sernum) == m_nCommPort;
}

CSiUtilEC3::CSiUtilEC3(const char* pSerNum)
{
    // There is always enough memory available, so there
    // is no need to check the value returned by _strdup. ;-)
    m_pSerNum = (IS_INTRESOURCE(pSerNum)) ? nullptr : _strdup(pSerNum);
}

CSiUtilEC3::~CSiUtilEC3()
{
    ::DisconnectUSB();
    free(m_pSerNum);
}

HRESULT
CSiUtilEC3::Connect()
{
    if(::Connected())
        return S_OK;

    const HRESULT hr = ::ConnectUSB(m_pSerNum, 1, FALSE, FALSE);

    if(FAILED(hr))
        MessageBoxA(nullptr, ::GetErrorMsg(hr), nullptr, MB_OK);

    return(hr);
}

HRESULT
CSiUtilEC3::Disconnect()
{
    if(!::Connected())
        return S_OK;

    return ::DisconnectUSB();
}

bool
CSiUtilEC3::operator==(const char* sernum)
{
    if(IS_INTRESOURCE(sernum))
        return false;

    return !strcmp(sernum, m_pSerNum);
}
