#include "precomp.h"
#pragma intrinsic(memcmp)

INT_PTR
CSmartRF04EBDialog::OnWmInitDialog(HWND hWnd, WPARAM, LPARAM)
{
    TCHAR sz[MAX_PATH * 2];

    LoadString(g_hInst, IDS_CUEBANNER, sz, _countof(sz));
    SendDlgItemMessage(hWnd, IDC_DEBUGADAPTER, CB_SETCUEBANNER, 0, reinterpret_cast<LPARAM>(sz));
    InitializeDebugAdaptersCB(GetDlgItem(hWnd, IDC_DEBUGADAPTER));
    SendDlgItemMessage(hWnd, IDC_EBID, EM_LIMITTEXT, 4, 0);
    SetDlgItemInt(hWnd, IDC_EBID, 50, FALSE);
    EmbedControl(GetDlgItem(hWnd, IDC_BOOTLOADER), GetDlgItem(hWnd, IDC_BOOTLOADER_BROWSE));
    EmbedControl(GetDlgItem(hWnd, IDC_IMAGE), GetDlgItem(hWnd, IDC_IMAGE_BROWSE));

    if(GetSrf04EbFirmwareDir(sz, _countof(sz)))
    {
        MyPathCchAppend(sz, _countof(sz), TEXT("srf04eb_bootloader.hex"));

        if(PathFileExists(sz))
            SetDlgItemText(hWnd, IDC_BOOTLOADER, sz);

        PathRemoveFileSpec(sz);
        MyPathCchAppend(sz, _countof(sz), TEXT("fw0400.hex"));

        if(PathFileExists(sz))
            SetDlgItemText(hWnd, IDC_IMAGE, sz);
    }

    return TRUE;
}

VOID
CSmartRF04EBDialog::DoDownload(HWND hWndParent)
{
    CSiUtil*    pSiUtil;
    CCursor     cursor(IDC_WAIT);
    TCHAR       szId[8];
    char        szFileName[MAX_PATH * 2];
    int         idx;

    if(CB_ERR == (idx = static_cast<int>(SendDlgItemMessage(hWndParent, IDC_DEBUGADAPTER, CB_GETCURSEL, 0, 0))))
        return;

    const char* pSerNum = reinterpret_cast<const char*>(SendDlgItemMessage(hWndParent, IDC_DEBUGADAPTER, CB_GETITEMDATA, static_cast<WPARAM>(idx), 0));
    CSiUtilEC2 ec2(PtrToInt(pSerNum));
    CSiUtilEC3 ec3(pSerNum);

    if(IS_INTRESOURCE(pSerNum))
        pSiUtil = &ec2;
    else
        pSiUtil = &ec3;

    if(GetWindowTextA(hWndParent, szFileName, _countof(szFileName)))
        pSiUtil->SetAppName(szFileName);

    GetDlgItemTextA(hWndParent, IDC_BOOTLOADER, szFileName, _countof(szFileName));

    if(FAILED(pSiUtil->Download(szFileName, TRUE)))
        return;

    cursor.Refresh();
    GetDlgItemTextA(hWndParent, IDC_IMAGE, szFileName, _countof(szFileName));

    if(FAILED(pSiUtil->Download(szFileName, FALSE)))
        return;

    cursor.Refresh();
    MarkDeviceAsBusPowered(pSiUtil);
    cursor.Refresh();

    if(GetDlgItemText(hWndParent, IDC_EBID, szId, _countof(szId)))
    {
        unsigned long ebid = _tcstoul(szId, nullptr, 16);
        pSiUtil->SetCodeMemory(reinterpret_cast<BYTE*>(&ebid), 0x008A, 2);
    }

    cursor.Refresh();
    pSiUtil->SetTargetGo();
    SelectDebugAdapter(GetDlgItem(hWndParent, IDC_DEBUGADAPTER), *pSiUtil);
}

VOID
CSmartRF04EBDialog::MarkDeviceAsBusPowered(CSiUtil* pSiUtil)
{
    // Buggy CONFIGURATION_DESCRIPTOR (bmAttributes.7 is set to 0)
    static const BYTE bDefaultConfiguration1[] =
    {
        0x09, 0x02, 0x20, 0x00, 0x01, 0x01, 0x00, 0x40,
        0x32, 0x09, 0x04, 0x00, 0x00, 0x02, 0xFF, 0xFF,
        0xFF, 0x00, 0x07, 0x05, 0x82, 0x02, 0x40, 0x00,
        0x05, 0x07, 0x05, 0x02, 0x02, 0x40, 0x00, 0x05
    };

    // Correct CONFIGURATION_DESCRIPTOR (bmAttributes.7 set to 1)
    static const BYTE bDefaultConfiguration2[] =
    {
        0x09, 0x02, 0x20, 0x00, 0x01, 0x01, 0x00, 0xC0,
        0x32, 0x09, 0x04, 0x00, 0x00, 0x02, 0xFF, 0xFF,
        0xFF, 0x00, 0x07, 0x05, 0x82, 0x02, 0x40, 0x00,
        0x05, 0x07, 0x05, 0x02, 0x02, 0x40, 0x00, 0x05
    };

    BYTE bMem[0x3E00];  // 16 KB minus reserved page
    DWORD addr;

    if(FAILED(pSiUtil->GetCodeMemory(bMem, 0x0000, sizeof(bMem))))
        return;

    for(addr = 0; addr < (sizeof(bMem) - sizeof(bDefaultConfiguration1)); ++addr)
    {
        if(!memcmp(&bMem[addr], bDefaultConfiguration1, sizeof(bDefaultConfiguration1)))
        {
            unsigned bmAttributes = addr + 7;
            bMem[bmAttributes] &= ~0x40;    // mark as bus powered
            bMem[bmAttributes] |= 0x80;     // set bmAttributes.7
            pSiUtil->SetCodeMemory(&bMem[bmAttributes], addr, 1);
            return;
        }
    }

    // Buggy CONFIGURATION_DESCRIPTOR not found, see if it has been fixed by TI
    for(addr = 0; addr < (sizeof(bMem) - sizeof(bDefaultConfiguration2)); ++addr)
    {
        if(!memcmp(&bMem[addr], bDefaultConfiguration2, sizeof(bDefaultConfiguration2)))
        {
            unsigned bmAttributes = addr + 7;
            bMem[bmAttributes] &= ~0x40;    // mark as bus powered
            pSiUtil->SetCodeMemory(&bMem[bmAttributes], addr, 1);
            return;
        }
    }
}

INT_PTR
CSmartRF04EBDialog::OnWmCommand(HWND hWnd, WPARAM wParam, LPARAM)
{
    switch(LOWORD(wParam))
    {
        case IDC_BOOTLOADER_BROWSE:
        case IDC_IMAGE_BROWSE:
            AskForFileName(hWnd, LOWORD(wParam) - 1, IDS_FILTER, GUID_SmartRF04EB);
            return(TRUE);
        case IDOK:
            DoDownload(hWnd);
            return TRUE;
        case IDCANCEL:
            EndDialog(hWnd, 0);
            return TRUE;
        default:
            break;
    }

    return FALSE;
}

INT_PTR
CSmartRF04EBDialog::OnWmDeviceChange(HWND hWnd, WPARAM, LPARAM)
{
    InitializeDebugAdaptersCB(GetDlgItem(hWnd, IDC_DEBUGADAPTER));
    return 0;
}

INT_PTR CALLBACK
CSmartRF04EBDialog::DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    static const DLG_MSGMAP<CSmartRF04EBDialog> sc_map[] =
    {
        { WM_COMMAND,       &CSmartRF04EBDialog::OnWmCommand        },
        { WM_DEVICECHANGE,  &CSmartRF04EBDialog::OnWmDeviceChange   },
        { WM_DPICHANGED,    &CSmartRF04EBDialog::OnWmDeviceChange   }
    };

    for(uintptr_t idx = 0; idx < _countof(sc_map); ++idx)
    {
        if(sc_map[idx].uMsg == uMsg)
        {
            CSmartRF04EBDialog* pThis = reinterpret_cast<CSmartRF04EBDialog*>(GetWindowLongPtr(hWnd, DWLP_USER));

            if(pThis)
                return (pThis->*sc_map[idx].pfn)(hWnd, wParam, lParam);
        }
    }

    if(uMsg == WM_INITDIALOG)
    {
        SetWindowLongPtr(hWnd, DWLP_USER, lParam);
        return reinterpret_cast<CSmartRF04EBDialog*>(lParam)->OnWmInitDialog(hWnd, wParam, lParam);
    }

    return FALSE;
}

int
CSmartRF04EBDialog::DoModal(HWND hWndParent)
{
    CSmartRF04EBDialog dlg;
    return static_cast<int>(DialogBoxParam(g_hInst, MAKEINTRESOURCE(IDD_SMARTRF04EB), hWndParent, DlgProc, reinterpret_cast<LPARAM>(&dlg)));
}
